import click
import logging
from aws_profile_manager import Common

@click.command()
def cli():
    """ Shows all AWS profile from your ~/.aws/credentials """
    common = Common()

    users_home = common.get_users_home()
    all_profiles = common.get_all_profiles(users_home)
    _i = 0
    for _p in all_profiles:
        if _p == 'default' and len(all_profiles) > 1:
            for i in all_profiles:
                if i == 'default':
                    continue
                elif all_profiles[i] == all_profiles[_p]:
                    _p = f'{_p} (same as {i})'
                    break
        _i += 1
        print(f'{_i}. {_p}')


import sys
import click
import logging

from aws_profile_manager import Common, Rotate

@click.command()
@click.option('--aws-profile-name', required=False, help="AWS profile to rotate keys", type=str)
@click.option('--yes', is_flag=True, required=False, help="Confirm that you agree to rotate keys. Avoids manual confirmation", default=False, type=bool)
def cli(aws_profile_name, yes):
    """ Rotate keys for defined profile ~/.aws/credentials """
    common = Common()
    users_home = common.get_users_home()
    all_profiles = common.get_all_profiles(users_home)
    if not aws_profile_name:
        aws_profile_name = common.choose_profile(all_profiles, operation="rotate")
    access_key_id = all_profiles.get(aws_profile_name).get('aws_access_key_id')
    rotate = Rotate()
    current_access_keys = rotate.get_access_keys()
    if len(current_access_keys) > 1:
        logging.warning('There is more than one access key is created for the current user. '
        f'One (which is not used in profile [{aws_profile_name}]) should be deleted. '
        'Otherwise, it is not possible to make a credentials rotation')
        for _ak in current_access_keys:
            if _ak != access_key_id:
                while True:
                    if yes:
                        answer = 'yes'
                    else:
                        _ak_last_used = rotate.get_access_key_last_used(_ak)
                        answer = input(f"\nAccess key {_ak} (last used: {_ak_last_used['LastUsedDate'] if 'LastUsedDate' in _ak_last_used else 'N/A'}) will be deleted. Do you agree with that? [Y/n] ")
                    if answer.lower() == "yes" or answer.lower() == "y":
                        rotate.delete_access_key(access_key_id=_ak)
                        break
                    elif answer.lower() == "no" or answer.lower() == "n":
                        print('Exit...')
                        sys.exit(1)
                    else:
                        print("Yes or No?")
    aws_access_key_id, aws_secret_access_key = rotate.create_access_key()
    if aws_access_key_id and aws_secret_access_key:
        rotate.delete_access_key(access_key_id=access_key_id)
        all_profiles[aws_profile_name]['aws_access_key_id'] = aws_access_key_id
        all_profiles[aws_profile_name]['aws_secret_access_key'] = aws_secret_access_key
        common.rewrite_credentials_file(all_profiles, users_home)



import sys
import logging
from typing import List, Tuple
import boto3
from botocore import exceptions

class Rotate: #meybe Rotate sounds better
    def __init__(self) -> None:
        """ Class constructor """
        try:
            self.username = boto3.client('sts').get_caller_identity().get("Arn").split("/")[-1]
            self.iam_client = boto3.client('iam')
        except exceptions.ClientError:
            logging.error("There is some issue with connection to the AWS using your current credentials. Exit...")
            sys.exit(1)

    def get_access_keys(self) -> List:
        """ Method reaches AWS IAM service to get data about users AWS Access Keys.
            Returns list of Access Keys assigned to user in IAM """
        _access_keys_data = self.iam_client.list_access_keys(UserName=self.username).get('AccessKeyMetadata')
        _access_keys = [ i.get('AccessKeyId') for i in _access_keys_data ]
        return _access_keys

    def delete_access_key(self, access_key_id: str) -> bool:
        """ Method allows to deletes Access Keys for defined user """
        try:
            response = self.iam_client.delete_access_key(
                UserName=self.username,
                AccessKeyId=access_key_id
            )
        except:
            logging.error(f"Something went wrong during the {access_key_id} access key removal")
            return response
        else:
            logging.info(f"Access key {access_key_id} has been successfully removed")
        return

    def create_access_key(self) -> Tuple[str, str]:
        """ Method allows to create Access Keys for defined user """
        try:
            response = self.iam_client.create_access_key(
                UserName=self.username
            )
        except:
            logging.error(f"Something went wrong during the process of new credentials creation")
            return (False, False)
        else:
            logging.info(f"New credentials for user {self.username} have been successfully created")
        return (response.get('AccessKey').get('AccessKeyId'), response.get('AccessKey').get('SecretAccessKey'))
        
    def get_access_key_last_used(self, access_key_id: str) -> str:
        """ Method returns data about when and how Access Key was used """
        try:
            response = self.iam_client.get_access_key_last_used(
                AccessKeyId=access_key_id
            )
        except:
            logging.error(f"Couldn't get info about access key {access_key_id}")
            return ''
        else:
            logging.info(f"Last time the key {access_key_id} was used on {response.get('AccessKeyLastUsed').get('LastUsedDate')} in the {response.get('AccessKeyLastUsed').get('Region')} region")
        return response.get('AccessKeyLastUsed')




import sys
import click
import logging
from aws_profile_manager import Common, Add

@click.command()
@click.option('--aws-profile-name', required=False, help="New AWS profile name", type=str)
@click.option('--aws-access-key-id', required=False, help="AWS Access Key ID", type=str)
@click.option('--aws-secret-access-key', required=False, help="AWS Secret Access Key", type=str)
def cli(aws_profile_name, aws_access_key_id, aws_secret_access_key):
    """ Add new AWS profile to your ~/.aws/credentials """
    common = Common()
    add = Add()
    users_home = common.get_users_home()
    all_profiles = common.get_all_profiles(users_home)
    if not aws_profile_name:
        aws_profile_name = add.ask_profile_name()
    if not aws_access_key_id:
        while True:
            aws_access_key_id = add.ask_aws_access_key_id()
            if not common.aws_access_key_id_is_valid(aws_access_key_id):
                logging.error("Invalid AWS_ACCESS_KEY_ID format. Please, try again")
            else:
                break
    if not aws_secret_access_key:
        while True:
            aws_secret_access_key = add.ask_aws_secret_access_key()
            if not common.aws_secret_access_key_is_valid(aws_secret_access_key):
                logging.error("Invalid AWS_SECRET_ACCESS_KEY format. Please, try again")
            else:
                break
    all_profiles[aws_profile_name] = { 
        'aws_access_key_id': aws_access_key_id, 
        'aws_secret_access_key': aws_secret_access_key
        }
    common.rewrite_credentials_file(all_profiles, users_home)



import sys
import logging

class Add:
    def ask_profile_name(self):
        """ TBD """
        aws_profile_name = input("New profile name: ")
        return aws_profile_name

    def ask_aws_access_key_id(self):
        """ TBD """
        aws_access_key_id = input("AWS Access Key ID: ")
        return aws_access_key_id
        
    def ask_aws_secret_access_key(self):
        """ TBD """
        aws_secret_access_key = input("AWS Secret Access Key: ")
        return aws_secret_access_key




import sys
import click
import logging
from aws_profile_manager import Common

@click.command()
@click.option('--aws-profile-name', required=False, help="AWS profile to remove", type=str)
@click.option('--yes', is_flag=True, required=False, help="Confirm that you agree to remove keys. Avoids manual confirmation", default=False, type=bool)
def cli(aws_profile_name, yes):
    """ Removes AWS profile from your ~/.aws/credentials """
    common = Common()

    users_home = common.get_users_home()
    all_profiles = common.get_all_profiles(users_home)
    if not aws_profile_name:
        aws_profile_name = common.choose_profile(all_profiles, operation="remove")
    try:
        while True:
            if yes:
                answer = 'yes'
            else:
                answer = input("Profile [{}] will be removed. Are you sure you want to proceed? [Y/n] ".format(aws_profile_name))
            if answer.lower() == "yes" or answer.lower() == "y":
                del all_profiles[aws_profile_name]
                logging.info("Profile {} has been successfuly removed from {}/.aws/credentials".format(aws_profile_name, users_home))
                break
            elif answer.lower() == "no" or answer.lower() == "n":
                break
            else:
                print("Yes or No?")
    except KeyError as e:
        logging.error("Looks like profile {} does not exist in {}/.aws/credentials".format(e, users_home))
        sys.exit(1)
    common.rewrite_credentials_file(all_profiles, users_home)


import unittest
from aws_profile_manager import Switch

class TestCommon(unittest.TestCase):
    def setUp(self):
        self.switch = Switch()

    def test_get_defaults_backup_without_default_backup(self):
        test_data = {
                    'default': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'
                        },
                    'test1': 
                        {
                            'aws_access_key_id': 'AKIAZZZZZZZZZZZZZZZZ',
                            'aws_secret_access_key': 'zZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZ'}
                    }
        expected_output = None
        self.assertEquals(self.switch.get_defaults_backup(test_data), expected_output)

    def test_get_defaults_backup_with_default_backup(self):
        test_data = {
                    'default': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'
                        },
                    'test1': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'}
                    }
        expected_output = 'test1'
        self.assertEquals(self.switch.get_defaults_backup(test_data), expected_output)

    def test_create_backup_for_default(self):
        test_data = {
                    'default': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'
                        },
                    'test1': 
                        {
                            'aws_access_key_id': 'AKIAZZZZZZZZZZZZZZZZ',
                            'aws_secret_access_key': 'zZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZ'}
                    }
        expected_output = {
                    'default': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'
                        },
                    'test1': 
                        {
                            'aws_access_key_id': 'AKIAZZZZZZZZZZZZZZZZ',
                            'aws_secret_access_key': 'zZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZzZ'},
                    'test2': 
                        {
                            'aws_access_key_id': 'AKIAYYYYYYYYYYYYYYYY',
                            'aws_secret_access_key': 'yYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyYyY'}
                    }
        self.assertDictEqual(self.switch.create_backup_for_default(test_data, 'test2'), expected_output)

if __name__ == '__main__':
    unittest.main()



from aws_profile_manager._version import __version__
from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        README = f.read()
    return README


setup(
    name='aws-profile-manager',
    version=__version__,
    description='This util allows you to manager your AWS profiles like add, remove, update and switch default AWS CLI profile',
    long_description=readme(),
    long_description_content_type="text/markdown",
    url='https://github.com/99stealth/aws-profile-manager',
    author='Roman Banakh',
    author_email='banakh.ri@gmail.com',
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    packages=find_packages(exclude=['aws-profile-manager.tests']),
    include_package_data=True,
    install_requires=['configparser', 'argparse', 'click', 'boto3'],
    entry_points={
        'console_scripts': [
            'aws-profile-manager=aws_profile_manager.cli:cli'
        ]
    }
)


import sys
from typing import Dict
import logging

class Switch:
    def get_defaults_backup(self, profiles: Dict):
        """ The function receives all profiles and looks for duplication of default. 
            It returns default if one is found and None if not """

        for profile in profiles:
            if profile == 'default':
                continue
            if profiles[profile] == profiles['default']:
                return profile
        return None

    def ask_new_name_for_default_profile(self, users_home: str, profiles: Dict) -> None:
        """ The function receives user_home and profiles
            It uses users_home only for notification and profiles for backing up a current default profile 
            The function returns a backup name for current backup if the user agreed to back it up 
            otherwise it returns None """

        print("There is no default profile duplication among all accounts in {}/.aws/credentials".format(users_home))
        while True:
            try:
                answer = input("Do you want to make backup of your current [default] profile? [Y/n] ")
                if answer.lower() == "yes" or answer.lower() == "y":
                    new_default_name = input("Enter new name for your currnet [default] profile: ")
                    if new_default_name in profiles:
                        print("Profile with name {} is already exists in {}/.aws/credentials. Try once more".format(new_default_name, users_home))
                        continue
                    else:
                        return new_default_name
                elif answer.lower() == "no" or answer.lower() == "n":
                    while True:
                        answer = input("Are you sure? Current [default] will be lost [Y/n] ")
                        if answer.lower() == "yes" or answer.lower() == "y":
                            return None
                        elif answer.lower() == "no" or answer.lower() == "n":
                            break
                        else:
                            print("Yes or No?")
                else:
                    print("Yes or No?")
            except (KeyboardInterrupt, EOFError):
                logging.error("\nProcess has been stopped. Interrupted by user")
                sys.exit(1)

    def create_backup_for_default(self, profiles: Dict, new_default_profile_name: str) -> Dict:
        profiles[new_default_profile_name] = profiles["default"]
        return profiles

    def choose_new_default(self, profiles: Dict, new_default_profile_name: str) -> str:
        i = 1
        counter = {}
        for profile in profiles:
            if profile == "default" or profile == new_default_profile_name:
                continue
            counter[i] = profile
            i += 1
        for c in counter:
            print("{}: {}".format(c, counter[c]))
        while True:
            try:
                answer = input("\nChoose a number of the account to which you want to switch your current default: ")
            except (KeyboardInterrupt, EOFError):
                logging.error("\nProcess has been stopped. Interrupted by user")
                sys.exit(1)
            try:
                return counter[int(answer)]
            except ValueError as e:
                logging.error("Answer \"{}\" is not valid. Please provide number from 1 to {}".format(answer, len(counter)))